
# q-expansion of newform 240.2.y.d, downloaded from the LMFDB on 01 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 240
weight = 2
poly_data = [8, -8, 6, -6, 3, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[-8, 2, -4, 3, 0, 1], 4], [[8, -6, 6, -3, 2, -1], 4], [[6, -2, 3, -3, 1, -1], 2], [[-12, 6, -6, 5, -2, 3], 4], [[12, -2, 10, -5, 2, -3], 4]]

hecke_ring_character_values = [[31, [-1, 0, 0, 0, 0, 0]], [181, [0, 1, 0, 0, 0, 0]], [161, [1, 0, 0, 0, 0, 0]], [97, [0, 1, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 1, 0, 0], [1, 0, 0, 0, 0, 0], [1, 2, 0, 0, 0, 0], [0, 0, -1, -1, 1, 1], [0, 0, 1, -1, -1, 1], [0, 0, 0, 0, 0, 0], [1, -1, -2, 0, -2, 0], [-2, 0, 1, -3, -1, -1], [2, 0, -1, 3, 1, 1], [-1, -1, 0, 0, 0, 0], [0, 0, -2, 0, 2, 0], [2, 2, -2, 2, -2, 2], [0, -4, 0, 0, 0, 0], [-2, -2, 4, -2, 0, -2], [2, -4, 3, -5, -3, 1], [-4, 2, 2, 6, 2, -2], [-4, 0, 5, 1, 3, -1], [-3, 1, 0, -2, 0, -2], [-2, -6, 0, -2, 4, -2], [-4, 0, 0, 4, 0, -4], [1, -3, -4, 0, -4, 0], [4, 0, 0, -4, 0, 4], [-8, 0, 0, 4, 0, -4], [-4, -2, -2, -2, -2, -2], [-3, -3, 4, 4, -4, -4], [-5, -1, 4, 0, 4, 0], [4, 0, -1, 5, 1, 3], [4, 4, 4, 8, 4, 0], [7, 5, -6, -4, 2, 4], [3, -1, 0, 2, 0, 2], [4, -4, -1, 1, 1, -1], [8, 4, -3, 1, -5, -1], [7, -9, 4, -6, -4, 2], [4, 2, -1, 1, -3, -1], [-1, -3, 0, -4, 0, -4], [0, 0, 0, 0, 0, 0], [-10, 4, 4, 4, 4, 4], [-8, 0, 0, -4, 0, 4], [4, -2, -7, -1, -5, 1], [-4, 0, 4, -4, 4, -4], [0, 4, 1, 3, -1, 5], [-1, 1, 2, 0, 2, 0], [-2, 14, 4, -2, 0, -2], [1, -1, -4, 4, 4, -4], [-2, -2, -2, -2, 6, -2], [0, 12, 2, 0, -2, 0], [4, -2, -9, -3, -3, 3], [-12, -8, 3, -1, 5, 1], [0, 8, -2, 0, 2, 0], [5, -7, 4, -6, -4, 2], [5, 3, -6, -4, 2, 4], [-4, 0, 0, -4, 0, 4], [2, 0, 0, -8, 0, 8], [-4, 4, 1, -1, -1, 1], [13, 7, -10, -4, -2, 4], [-10, 12, 3, -1, -3, 5], [7, 3, 0, 4, -8, -4], [0, 12, 2, 0, -2, 0], [2, 10, -10, 2, 6, 2], [-2, 10, 2, -2, 2, -2], [-2, -14, 8, -2, -4, -2], [6, -4, -4, -12, -4, 4], [-2, 14, 4, -2, 0, -2], [8, 4, 4, 8, 4, 0], [-3, -3, 0, 0, 0, 0], [-12, 2, 2, 6, 2, -2], [2, 4, -3, 9, 3, 3], [-15, -15, 4, 4, -4, -4], [0, 4, 4, 4, 4, 4], [-5, -11, -10, -4, -2, 4], [-1, -1, 0, -2, 0, -2], [-6, -10, 12, -6, 0, -6], [-8, 12, -5, 9, 5, -1], [-2, -4, -4, -12, -4, 4], [4, 2, -9, -7, 5, 7], [12, 10, -3, -1, -1, 1], [7, -11, 0, -4, 0, -4], [8, 2, 2, 2, 2, 2], [0, 2, 2, 10, 2, -6], [-16, 6, 6, 10, 6, 2], [0, 4, -3, 7, 3, 1], [-17, -15, 6, 4, -2, -4], [-2, -22, 0, -2, 4, -2], [1, 3, -4, 8, 4, 0], [-4, 8, 6, -4, 2, -4], [4, 20, 2, 4, -10, 4], [6, 10, -6, 6, -6, 6], [5, -5, -4, 4, 4, -4], [-5, 1, 4, -8, -4, 0], [8, 12, 3, -1, 5, 1], [-4, -8, 6, -4, 2, -4], [-20, 0, 0, 4, 0, -4], [4, 4, -9, -9, 9, 9], [12, -16, 1, -5, -1, -3], [-2, 0, 1, -3, -1, -1], [-6, 0, -5, -1, 5, -11], [-9, -9, 8, 8, -8, -8], [6, -14, -6, 6, -6, 6], [-2, 2, 0, -2, 4, -2], [-7, -7, 4, -18, -4, -10], [-2, -26, 4, -2, 0, -2], [10, 0, 0, 8, 0, -8], [-8, -4, -4, -4, -4, -4], [-16, 2, 2, 2, 2, 2], [-18, 20, 1, 1, -1, 3], [21, 21, -4, -4, 4, 4], [-16, 0, 0, 4, 0, -4], [3, -5, 8, -10, -8, 6], [-6, -2, 12, -6, 0, -6], [8, 8, -24, 8, 8, 8], [12, 0, -5, 17, 5, 7], [16, 2, 2, 2, 2, 2], [-13, 3, 4, -14, -4, -6], [4, 2, -1, 1, -3, -1], [12, -8, -8, -12, -8, -4], [18, 0, 0, 8, 0, -8], [8, -8, -8, -12, -8, -4], [0, 2, -7, -9, 11, 9], [-2, -10, 6, -2, -2, -2], [8, 0, -3, 11, 3, 5], [11, 21, 14, 4, 6, -4], [5, 3, 8, 0, -8, 16], [-2, -10, -2, -2, 6, -2], [4, -16, -10, 4, 2, 4], [16, 22, 7, 1, 5, -1], [7, -3, 4, 0, -4, 8], [17, -19, 0, -2, 0, -2], [-8, 0, 0, 0, 0, 0], [12, -4, -17, -1, -15, 1], [6, -2, -6, 6, -6, 6], [-22, 20, -3, 1, 3, -5], [14, -8, -1, 7, 1, 5], [-4, 12, 2, -4, 6, -4], [6, 14, -14, 6, 2, 6], [2, 6, 6, 2, -10, 2], [-2, 10, -2, -2, 6, -2], [-4, 10, 10, 22, 10, -2], [6, 26, 0, 6, -12, 6], [-4, -6, -6, -18, -6, 6], [10, -8, -8, 0, -8, -16], [22, -16, -3, 9, 3, 3], [19, 7, -12, 0, -12, 0], [8, 8, -9, 25, 9, 7], [-8, 4, 4, 12, 4, -4], [-1, 5, 2, -4, 10, 4], [6, 26, -16, 6, 4, 6], [-36, -2, -2, -2, -2, -2], [-5, 3, -4, 2, 4, -6], [-12, 6, 31, 13, 5, -13], [0, 2, 5, 3, -1, -3], [14, 4, 4, 4, 4, 4], [-12, 6, 6, 14, 6, -2], [20, -4, -4, 8, -4, -16], [-8, -10, -3, -1, -1, 1], [8, 4, 4, 4, 4, 4], [2, -10, -12, 2, 8, 2], [4, 12, 2, 4, -10, 4], [-10, 2, 26, -10, -6, -10], [1, -9, 8, -16, -8, 0], [-5, 5, -4, 4, 4, -4], [-4, -4, 18, -4, -10, -4], [-19, -21, -2, 0, -2, 0], [-12, -8, 19, 15, -11, -15], [-16, 4, 5, -17, -5, -7], [-19, -13, 10, 4, 2, -4], [2, 8, -5, 15, 5, 5], [-8, -20, 18, -8, -2, -8], [-4, -4, 12, -4, -4, -4]]
